//=============================================================================
// RPG Maker MZ - Alternative Menu Screen
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Alternative menu screen layout.
 * @author Yoji Ojima
 *
 * @help AltMenuScreen.js
 *
 * This plugin changes the layout of the menu screen.
 * It puts the commands on the top and the status on the bottom.
 *
 * It does not provide plugin commands.
 */

/*:ja
 * @target MZ
 * @plugindesc メニュー画面のレイアウトを変更します。
 * @author Yoji Ojima
 * @author Yamido
 *
 * @help AltMenuScreen.js
 *
 * このプラグインは、メニュー画面のレイアウトを変更します。
 * コマンドを上側に、ステータスを下側に配置します。
 *
 * プラグインコマンドはありません。
 */

(() => {
    Scene_MenuBase.prototype.commandWindowHeight = function() {
        return this.calcWindowHeight(6, true);
    };

    Scene_MenuBase.prototype.goldWindowHeight = function() {
        return 0;
    };
    Scene_MenuBase.prototype.statusWindowHeight = function() {
        return 0;
    };

    Scene_Menu.prototype.commandWindowRect = function() {
        const ww = Graphics.boxWidth / 2;
        const wh = this.commandWindowHeight();
        const wx = ww / 2;
        const wy = this.mainAreaTop() + (wh / 2);
        return new Rectangle(wx, wy, ww, wh);
    };

    Scene_Menu.prototype.goldWindowRect = function() {
        return new Rectangle(0,0,0,0);
    };
    Scene_Menu.prototype.statusWindowRect = function() {
        return new Rectangle(0,0,0,0);
    };

    Window_MenuCommand.prototype.maxCols = function() {
        return 1;
    };

    Window_MenuCommand.prototype.numVisibleRows = function() {
        return 8;
    };
    
    // 画面に見えなくても並列実行させる
    const _isNearTheScreen = Game_CharacterBase.prototype.isNearTheScreen;
	Game_CharacterBase.prototype.isNearTheScreen = function() {
		return (this._moveType === 3) || _isNearTheScreen.call(this);
	};
	
	/*
	const _lp = Sprite_ExtraGaugeContainer.prototype.updateLongPressOnMax;
    Sprite_ExtraGaugeContainer.prototype.updateLongPressOnMax = function() {
        _lp.call(this);
        this._longPressGaugeValue = 0; // Added by Yam (これを入れないと1度クリックしないと（onReleaseされないと）ゲージが反応しない)
    };*/

    // どうにか外部からonRelease呼ばないと...下のはダメ
    const _onRelase = Sprite_ExtraGaugeContainer.prototype.onRelease;
    Game_System.prototype.forceOnRelease = function(type) {
      _onRelase.call(this);
    }
    
    
    //探索センサー視界On/Off
    Game_System.prototype.onAllSensorVisible = function() {
        $gameMap.events().forEach(function(event) {
             event.setRangeVisible(1);
        }, this);
    }
    
    Game_System.prototype.offAllSensorVisible = function() {
        $gameMap.events().forEach(function(event) {
             event.setRangeVisible(false);
        }, this);
    }
})();
